@@ -9,10 +9,10 @@ from dagster.cli.workspace.cli_target import (
     get_external_repository_from_kwargs,
     repository_target_argument,
 )
-from dagster.core.definitions.run_request import JobType
+from dagster.core.definitions.run_request import InstigatorType
 from dagster.core.host_representation import ExternalRepository
 from dagster.core.instance import DagsterInstance
-from dagster.core.scheduler.job import JobStatus
+from dagster.core.scheduler.instigation import InstigatorStatus
 from dagster.core.scheduler.scheduler import DagsterDaemonScheduler
 
 
@@ -27,11 +27,11 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
     debug_info = instance.scheduler_debug_info()
     errors = debug_info.errors
     external_schedules = external_repository.get_external_schedules()
-    schedule_states = instance.all_stored_job_state(
-        external_repository.get_external_origin_id(), JobType.SCHEDULE
+    schedule_states = instance.all_instigator_state(
+        external_repository.get_external_origin_id(), InstigatorType.SCHEDULE
     )
     external_schedules_dict = {s.get_external_origin_id(): s for s in external_schedules}
-    schedule_states_dict = {s.job_origin_id: s for s in schedule_states}
+    schedule_states_dict = {s.instigator_origin_id: s for s in schedule_states}
 
     external_schedule_origin_ids = set(external_schedules_dict.keys())
     schedule_state_ids = set(schedule_states_dict.keys())
@@ -43,7 +43,7 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
     for schedule_origin_id in external_schedule_origin_ids & schedule_state_ids:
         schedule_state = schedule_states_dict[schedule_origin_id]
         external_schedule = external_schedules_dict[schedule_origin_id]
-        if schedule_state.job_specific_data.cron_schedule != external_schedule.cron_schedule:
+        if schedule_state.instigator_data.cron_schedule != external_schedule.cron_schedule:
             changed_schedules.append(schedule_origin_id)
 
     if not errors and not added_schedules and not changed_schedules and not removed_schedules:
@@ -94,7 +94,7 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
         )
         print_fn(
             click.style("\t cron_schedule: ", fg="yellow")
-            + click.style(schedule_state.job_specific_data.cron_schedule, fg="red")
+            + click.style(schedule_state.instigator_data.cron_schedule, fg="red")
             + " => "
             + click.style(external_schedule.cron_schedule, fg="green")
         )
@@ -103,7 +103,8 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
         print_fn(
             click.style(
                 "  - {name} (delete) [{id}]".format(
-                    name=schedule_states_dict[schedule_origin_id].job_name, id=schedule_origin_id
+                    name=schedule_states_dict[schedule_origin_id].instigator_name,
+                    id=schedule_origin_id,
                 ),
                 fg="red",
             )
@@ -154,44 +155,6 @@ def execute_preview_command(cli_args, print_fn):
             print_changes(external_repo, instance, print_fn, preview=True)
 
 
-@schedule_cli.command(name="up")
-@click.option("--preview", help="Preview changes", is_flag=True, default=False)
-@repository_target_argument
-def schedule_up_command(preview, **kwargs):
-    """
-    Update the Dagster instance's representation of schedules.
-
-    After running this command, the internal representation will match the list of
-    ScheduleDefinitions defined in the repository.
-
-    \b
-    To preview what changes will be applied, one of the following commands can be used:
-        dagster schedule up --preview
-        dagster schedule preview
-
-    Note by default, new ScheduleDefinitions in the Dagster instance will not start running until
-    they are explicitly started.
-    """
-    return execute_up_command(preview, kwargs, click.echo)
-
-
-def execute_up_command(preview, cli_args, print_fn):
-    with DagsterInstance.get() as instance:
-        with get_external_repository_from_kwargs(
-            instance, version=dagster_version, kwargs=cli_args
-        ) as external_repo:
-            check_repo_and_scheduler(external_repo, instance)
-
-            print_changes(external_repo, instance, print_fn, preview=preview)
-            if preview:
-                return
-
-            try:
-                instance.reconcile_scheduler_state(external_repo)
-            except DagsterInvariantViolationError as ex:
-                raise click.UsageError(ex)
-
-
 @schedule_cli.command(
     name="list",
     help="List all schedules that correspond to a repository.",
@@ -220,33 +183,30 @@ def execute_list_command(running_filter, stopped_filter, name_filter, cli_args,
 
             repo_schedules = external_repo.get_external_schedules()
             stored_schedules_by_origin_id = {
-                stored_schedule_state.job_origin_id: stored_schedule_state
-                for stored_schedule_state in instance.all_stored_job_state(
-                    external_repo.get_external_origin_id(), job_type=JobType.SCHEDULE
+                stored_schedule_state.instigator_origin_id: stored_schedule_state
+                for stored_schedule_state in instance.all_instigator_state(
+                    external_repo.get_external_origin_id(), instigator_type=InstigatorType.SCHEDULE
                 )
             }
 
             first = True
 
             for external_schedule in repo_schedules:
-                stored_schedule_state = stored_schedules_by_origin_id.get(
-                    external_schedule.get_external_origin_id()
+                schedule_state = external_schedule.get_current_instigator_state(
+                    stored_schedules_by_origin_id.get(external_schedule.get_external_origin_id())
                 )
-                if running_filter and (
-                    not stored_schedule_state or stored_schedule_state.status == JobStatus.STOPPED
-                ):
+
+                if running_filter and not schedule_state.is_running:
                     continue
-                if stopped_filter and stored_schedule_state and JobStatus.RUNNING:
+                if stopped_filter and schedule_state.is_running:
                     continue
 
                 if name_filter:
                     print_fn(external_schedule.name)
                     continue
 
-                status = (
-                    stored_schedule_state.status if stored_schedule_state else JobStatus.STOPPED
-                )
-                schedule_title = f"Schedule: {external_schedule.name} [{status.value}]"
+                status = "RUNNING" if schedule_state.is_running else "STOPPED"
+                schedule_title = f"Schedule: {external_schedule.name} [{status}]"
                 if not first:
                     print_fn("*" * len(schedule_title))
 
@@ -295,7 +255,7 @@ def execute_start_command(schedule_name, all_flag, cli_args, print_fn):
             if all_flag:
                 for external_schedule in external_repo.get_external_schedules():
                     try:
-                        instance.start_schedule_and_update_storage_state(external_schedule)
+                        instance.start_schedule(external_schedule)
                     except DagsterInvariantViolationError as ex:
                         raise click.UsageError(ex)
 
@@ -307,9 +267,7 @@ def execute_start_command(schedule_name, all_flag, cli_args, print_fn):
             else:
                 try:
 
-                    instance.start_schedule_and_update_storage_state(
-                        external_repo.get_external_schedule(schedule_name)
-                    )
+                    instance.start_schedule(external_repo.get_external_schedule(schedule_name))
                 except DagsterInvariantViolationError as ex:
                     raise click.UsageError(ex)
 
@@ -332,8 +290,9 @@ def execute_stop_command(schedule_name, cli_args, print_fn, instance=None):
             check_repo_and_scheduler(external_repo, instance)
 
             try:
-                instance.stop_schedule_and_update_storage_state(
-                    external_repo.get_external_schedule(schedule_name).get_external_origin_id()
+                external_schedule = external_repo.get_external_schedule(schedule_name)
+                instance.stop_schedule(
+                    external_schedule.get_external_origin_id(), external_schedule
                 )
             except DagsterInvariantViolationError as ex:
                 raise click.UsageError(ex)
@@ -431,18 +390,19 @@ def execute_restart_command(schedule_name, all_running_flag, cli_args, print_fn)
             repository_name = external_repo.name
 
             if all_running_flag:
-                for schedule_state in instance.all_stored_job_state(
-                    external_repo.get_external_origin_id(), JobType.SCHEDULE
+                for schedule_state in instance.all_instigator_state(
+                    external_repo.get_external_origin_id(), InstigatorType.SCHEDULE
                 ):
-                    if schedule_state.status == JobStatus.RUNNING:
+                    if schedule_state.status == InstigatorStatus.RUNNING:
                         try:
                             external_schedule = external_repo.get_external_schedule(
-                                schedule_state.job_name
+                                schedule_state.instigator_name
                             )
-                            instance.stop_schedule_and_update_storage_state(
-                                schedule_state.job_origin_id
+                            instance.stop_schedule(
+                                schedule_state.instigator_origin_id,
+                                external_schedule,
                             )
-                            instance.start_schedule_and_update_storage_state(external_schedule)
+                            instance.start_schedule(external_schedule)
                         except DagsterInvariantViolationError as ex:
                             raise click.UsageError(ex)
 
@@ -453,17 +413,19 @@ def execute_restart_command(schedule_name, all_running_flag, cli_args, print_fn)
                 )
             else:
                 external_schedule = external_repo.get_external_schedule(schedule_name)
-                schedule_state = instance.get_job_state(external_schedule.get_external_origin_id())
-                if schedule_state != None and schedule_state.status != JobStatus.RUNNING:
+                schedule_state = instance.get_instigator_state(
+                    external_schedule.get_external_origin_id()
+                )
+                if schedule_state != None and schedule_state.status != InstigatorStatus.RUNNING:
                     click.UsageError(
                         "Cannot restart a schedule {name} because is not currently running".format(
-                            name=schedule_state.job_name
+                            name=schedule_state.instigator_name
                         )
                     )
 
                 try:
-                    instance.stop_schedule_and_update_storage_state(schedule_state.job_origin_id)
-                    instance.start_schedule_and_update_storage_state(external_schedule)
+                    instance.stop_schedule(schedule_state.instigator_origin_id, external_schedule)
+                    instance.start_schedule(external_schedule)
                 except DagsterInvariantViolationError as ex:
                     raise click.UsageError(ex)
 
