@@ -1,6 +1,7 @@
 import os
 import sys
 from contextlib import contextmanager
+from typing import TYPE_CHECKING, Dict, Generator, Iterable, List, Optional, Tuple, Union, cast
 
 import click
 from click import UsageError
@@ -10,7 +11,11 @@ from dagster.core.definitions.reconstructable import repository_def_from_target_
 from dagster.core.host_representation.external import ExternalRepository
 from dagster.core.host_representation.repository_location import RepositoryLocation
 from dagster.core.instance import DagsterInstance
-from dagster.core.origin import PipelinePythonOrigin, RepositoryPythonOrigin
+from dagster.core.origin import (
+    DEFAULT_DAGSTER_ENTRY_POINT,
+    PipelinePythonOrigin,
+    RepositoryPythonOrigin,
+)
 from dagster.core.workspace.context import WorkspaceRequestContext
 from dagster.core.workspace.load_target import (
     EmptyWorkspaceTarget,
@@ -23,10 +28,13 @@ from dagster.core.workspace.load_target import (
 from dagster.grpc.utils import get_loadable_targets
 from dagster.utils.hosted_user_process import recon_repository_from_origin
 
+if TYPE_CHECKING:
+    from dagster.core.workspace.context import WorkspaceProcessContext
+
 WORKSPACE_TARGET_WARNING = "Can only use ONE of --workspace/-w, --python-file/-f, --module-name/-m, --grpc-port, --grpc-socket."
 
 
-def _cli_load_invariant(condition, msg=None):
+def _cli_load_invariant(condition: object, msg=None) -> None:
     msg = (
         msg
         or "Invalid set of CLI arguments for loading repository/pipeline. See --help for details."
@@ -35,12 +43,12 @@ def _cli_load_invariant(condition, msg=None):
         raise UsageError(msg)
 
 
-def _check_cli_arguments_none(kwargs, *keys):
+def _check_cli_arguments_none(kwargs: Dict[str, object], *keys: str) -> None:
     for key in keys:
         _cli_load_invariant(not kwargs.get(key))
 
 
-def are_all_keys_empty(kwargs, keys):
+def are_all_keys_empty(kwargs: Dict[str, object], keys: Iterable[str]) -> bool:
     for key in keys:
         if kwargs.get(key):
             return False
@@ -52,7 +60,6 @@ WORKSPACE_CLI_ARGS = (
     "workspace",
     "python_file",
     "working_directory",
-    "empty_working_directory",
     "package_name",
     "module_name",
     "attribute",
@@ -63,7 +70,7 @@ WORKSPACE_CLI_ARGS = (
 )
 
 
-def created_workspace_load_target(kwargs):
+def get_workspace_load_target(kwargs: Dict[str, object]):
     check.dict_param(kwargs, "kwargs")
     if are_all_keys_empty(kwargs, WORKSPACE_CLI_ARGS):
         if kwargs.get("empty_workspace"):
@@ -77,7 +84,6 @@ def created_workspace_load_target(kwargs):
             kwargs,
             "python_file",
             "working_directory",
-            "empty_working_directory",
             "module_name",
             "package_name",
             "attribute",
@@ -85,7 +91,7 @@ def created_workspace_load_target(kwargs):
             "grpc_port",
             "grpc_socket",
         )
-        return WorkspaceFileTarget(paths=list(kwargs["workspace"]))
+        return WorkspaceFileTarget(paths=list(cast(Union[List, Tuple], kwargs.get("workspace"))))
     if kwargs.get("python_file"):
         _check_cli_arguments_none(
             kwargs,
@@ -97,8 +103,8 @@ def created_workspace_load_target(kwargs):
         )
         working_directory = get_working_directory_from_kwargs(kwargs)
         return PythonFileTarget(
-            python_file=kwargs.get("python_file"),
-            attribute=kwargs.get("attribute"),
+            python_file=check.str_elem(kwargs, "python_file"),
+            attribute=check.opt_str_elem(kwargs, "attribute"),
             working_directory=working_directory,
             location_name=None,
         )
@@ -106,29 +112,29 @@ def created_workspace_load_target(kwargs):
         _check_cli_arguments_none(
             kwargs,
             "package_name",
-            "working_directory",
-            "empty_working_directory",
             "grpc_host",
             "grpc_port",
             "grpc_socket",
         )
+        working_directory = get_working_directory_from_kwargs(kwargs)
         return ModuleTarget(
-            module_name=kwargs.get("module_name"),
-            attribute=kwargs.get("attribute"),
+            module_name=check.str_elem(kwargs, "module_name"),
+            attribute=check.opt_str_elem(kwargs, "attribute"),
+            working_directory=working_directory,
             location_name=None,
         )
     if kwargs.get("package_name"):
         _check_cli_arguments_none(
             kwargs,
-            "working_directory",
-            "empty_working_directory",
             "grpc_host",
             "grpc_port",
             "grpc_socket",
         )
+        working_directory = get_working_directory_from_kwargs(kwargs)
         return PackageTarget(
-            package_name=kwargs.get("package_name"),
-            attribute=kwargs.get("attribute"),
+            package_name=check.str_elem(kwargs, "package_name"),
+            attribute=check.opt_str_elem(kwargs, "attribute"),
+            working_directory=working_directory,
             location_name=None,
         )
     if kwargs.get("grpc_port"):
@@ -136,13 +142,12 @@ def created_workspace_load_target(kwargs):
             kwargs,
             "attribute",
             "working_directory",
-            "empty_working_directory",
             "grpc_socket",
         )
         return GrpcServerTarget(
-            port=kwargs.get("grpc_port"),
+            port=check.int_elem(kwargs, "grpc_port"),
             socket=None,
-            host=(kwargs.get("grpc_host") if kwargs.get("grpc_host") else "localhost"),
+            host=check.opt_str_elem(kwargs, "grpc_host") or "localhost",
             location_name=None,
         )
     elif kwargs.get("grpc_socket"):
@@ -150,50 +155,45 @@ def created_workspace_load_target(kwargs):
             kwargs,
             "attribute",
             "working_directory",
-            "empty_working_directory",
         )
         return GrpcServerTarget(
             port=None,
-            socket=kwargs.get("grpc_socket"),
-            host=(kwargs.get("grpc_host") if kwargs.get("grpc_host") else "localhost"),
+            socket=check.str_elem(kwargs, "grpc_socket"),
+            host=check.opt_str_elem(kwargs, "grpc_host") or "localhost",
             location_name=None,
         )
     else:
         _cli_load_invariant(False)
+        # necessary for pyright, does not understand _cli_load_invariant(False) never returns
+        assert False
 
 
 def get_workspace_process_context_from_kwargs(
-    instance: DagsterInstance, version: str, read_only: bool, kwargs
-):
+    instance: DagsterInstance, version: str, read_only: bool, kwargs: Dict[str, object]
+) -> "WorkspaceProcessContext":
     from dagster.core.workspace import WorkspaceProcessContext
 
     return WorkspaceProcessContext(
-        instance, created_workspace_load_target(kwargs), version=version, read_only=read_only
+        instance, get_workspace_load_target(kwargs), version=version, read_only=read_only
     )
 
 
 @contextmanager
-def get_workspace_from_kwargs(instance: DagsterInstance, version: str, kwargs):
+def get_workspace_from_kwargs(
+    instance: DagsterInstance, version: str, kwargs: Dict[str, object]
+) -> Generator[WorkspaceRequestContext, None, None]:
     with get_workspace_process_context_from_kwargs(
         instance, version, read_only=False, kwargs=kwargs
     ) as workspace_process_context:
         yield workspace_process_context.create_request_context()
 
 
-def python_target_click_options():
+def python_target_click_options(is_using_job_op_graph_apis: bool = False):
     return [
-        click.option(
-            "--empty-working-directory",
-            is_flag=True,
-            required=False,
-            default=False,
-            help="Indicates that the working directory should be empty and should not set to the current "
-            "directory as a default",
-        ),
         click.option(
             "--working-directory",
             "-d",
-            help="Specify working directory to use when loading the repository or pipeline. Can only be used along with -f/--python-file",
+            help=f"Specify working directory to use when loading the repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'}.",
         ),
         click.option(
             "--python-file",
@@ -201,21 +201,23 @@ def python_target_click_options():
             # Checks that the path actually exists lower in the stack, where we
             # are better equipped to surface errors
             type=click.Path(exists=False),
-            help="Specify python file where repository or pipeline function lives",
+            help=f"Specify python file where repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} function lives",
         ),
         click.option(
             "--package-name",
-            help="Specify installed Python package where repository or pipeline function lives",
+            help=f"Specify Python package where repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} function lives",
         ),
         click.option(
-            "--module-name", "-m", help="Specify module where repository or pipeline function lives"
+            "--module-name",
+            "-m",
+            help=f"Specify module where repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} function lives",
         ),
         click.option(
             "--attribute",
             "-a",
             help=(
-                "Attribute that is either a 1) repository or pipeline or "
-                "2) a function that returns a repository or pipeline"
+                f"Attribute that is either a 1) repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} or "
+                f"2) a function that returns a repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'}"
             ),
         ),
     ]
@@ -250,7 +252,7 @@ def grpc_server_target_click_options():
     ]
 
 
-def workspace_target_click_options():
+def workspace_target_click_options(using_job_op_graph_apis: bool = False):
     return (
         [
             click.option("--empty-workspace", is_flag=True, help="Allow an empty workspace"),
@@ -262,7 +264,7 @@ def workspace_target_click_options():
                 help=("Path to workspace file. Argument can be provided multiple times."),
             ),
         ]
-        + python_target_click_options()
+        + python_target_click_options(using_job_op_graph_apis)
         + grpc_server_target_click_options()
     )
 
@@ -283,7 +285,7 @@ def python_pipeline_target_click_options():
 
 def python_job_target_click_options():
     return (
-        python_target_click_options()
+        python_target_click_options(is_using_job_op_graph_apis=True)
         + [
             click.option(
                 "--repository",
@@ -347,6 +349,12 @@ def workspace_target_argument(f):
     return apply_click_params(f, *workspace_target_click_options())
 
 
+def job_workspace_target_argument(f):
+    from dagster.cli.pipeline import apply_click_params
+
+    return apply_click_params(f, *workspace_target_click_options(using_job_op_graph_apis=True))
+
+
 def grpc_server_origin_target_argument(f):
     from dagster.cli.pipeline import apply_click_params
 
@@ -361,11 +369,8 @@ def python_origin_target_argument(f):
     return apply_click_params(f, *options)
 
 
-def repository_target_argument(f):
-    from dagster.cli.pipeline import apply_click_params
-
-    return apply_click_params(
-        workspace_target_argument(f),
+def repository_click_options():
+    return [
         click.option(
             "--repository",
             "-r",
@@ -380,7 +385,19 @@ def repository_target_argument(f):
                 "RepositoryLocation within the workspace, necessary if more than one location is present."
             ),
         ),
-    )
+    ]
+
+
+def repository_target_argument(f):
+    from dagster.cli.pipeline import apply_click_params
+
+    return apply_click_params(workspace_target_argument(f), *repository_click_options())
+
+
+def job_repository_target_argument(f):
+    from dagster.cli.pipeline import apply_click_params
+
+    return apply_click_params(job_workspace_target_argument(f), *repository_click_options())
 
 
 def pipeline_option():
@@ -388,7 +405,9 @@ def pipeline_option():
         "--pipeline",
         "-p",
         "pipeline_or_job",
-        help=("Pipeline within the repository, necessary if more than one pipeline is present."),
+        help=(
+            "Pipeline/Job within the repository, necessary if more than one pipeline/job is present."
+        ),
     )
 
 
@@ -410,7 +429,7 @@ def job_option():
 def job_target_argument(f):
     from dagster.cli.pipeline import apply_click_params
 
-    return apply_click_params(repository_target_argument(f), job_option())
+    return apply_click_params(job_repository_target_argument(f), job_option())
 
 
 def get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_apis=False):
@@ -420,10 +439,12 @@ def get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_api
     recon_repo = recon_repository_from_origin(repository_origin)
     repo_definition = recon_repo.get_definition()
 
-    pipeline_names = set(repo_definition.pipeline_names)
+    pipeline_or_job_names = set(
+        repo_definition.job_names if using_job_op_graph_apis else repo_definition.pipeline_names
+    )
 
-    if provided_pipeline_name is None and len(pipeline_names) == 1:
-        pipeline_name = next(iter(pipeline_names))
+    if provided_pipeline_name is None and len(pipeline_or_job_names) == 1:
+        pipeline_name = next(iter(pipeline_or_job_names))
     elif provided_pipeline_name is None:
         raise click.UsageError(
             (
@@ -432,10 +453,10 @@ def get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_api
             ).format(
                 flag="--job" if using_job_op_graph_apis else "--pipeline",
                 repository=repo_definition.name,
-                pipelines=_sorted_quoted(pipeline_names),
+                pipelines=_sorted_quoted(pipeline_or_job_names),
             )
         )
-    elif not provided_pipeline_name in pipeline_names:
+    elif not provided_pipeline_name in pipeline_or_job_names:
         raise click.UsageError(
             (
                 'Pipeline/Job "{provided_pipeline_name}" not found in repository "{repository_name}". '
@@ -443,7 +464,7 @@ def get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_api
             ).format(
                 provided_pipeline_name=provided_pipeline_name,
                 repository_name=repo_definition.name,
-                found_names=_sorted_quoted(pipeline_names),
+                found_names=_sorted_quoted(pipeline_or_job_names),
             )
         )
     else:
@@ -471,21 +492,25 @@ def _get_code_pointer_dict_from_kwargs(kwargs):
             )
         }
     elif module_name:
-        _check_cli_arguments_none(kwargs, "python_file", "working_directory", "package_name")
+        _check_cli_arguments_none(kwargs, "python_file", "package_name")
         return {
             repository_def_from_target_def(
                 loadable_target.target_definition
-            ).name: CodePointer.from_module(module_name, loadable_target.attribute)
+            ).name: CodePointer.from_module(
+                module_name, loadable_target.attribute, working_directory
+            )
             for loadable_target in get_loadable_targets(
                 python_file, module_name, package_name, working_directory, attribute
             )
         }
     elif package_name:
-        _check_cli_arguments_none(kwargs, "module_name", "python_file", "working_directory")
+        _check_cli_arguments_none(kwargs, "module_name", "python_file")
         return {
             repository_def_from_target_def(
                 loadable_target.target_definition
-            ).name: CodePointer.from_python_package(package_name, loadable_target.attribute)
+            ).name: CodePointer.from_python_package(
+                package_name, loadable_target.attribute, working_directory
+            )
             for loadable_target in get_loadable_targets(
                 python_file, module_name, package_name, working_directory, attribute
             )
@@ -494,10 +519,8 @@ def _get_code_pointer_dict_from_kwargs(kwargs):
         check.failed("Must specify a Python file or module name")
 
 
-def get_working_directory_from_kwargs(kwargs):
-    if kwargs.get("empty_working_directory"):
-        return None
-    return kwargs.get("working_directory") if kwargs.get("working_directory") else os.getcwd()
+def get_working_directory_from_kwargs(kwargs: Dict[str, object]) -> Optional[str]:
+    return check.opt_str_elem(kwargs, "working_directory") or os.getcwd()
 
 
 def get_repository_python_origin_from_kwargs(kwargs):
@@ -519,20 +542,26 @@ def get_repository_python_origin_from_kwargs(kwargs):
                 get_working_directory_from_kwargs(kwargs),
             )
         elif kwargs.get("module_name"):
-            _check_cli_arguments_none(kwargs, "python_file", "working_directory", "package_name")
+            _check_cli_arguments_none(kwargs, "python_file", "package_name")
             code_pointer = CodePointer.from_module(
                 kwargs.get("module_name"),
                 kwargs.get("attribute"),
+                get_working_directory_from_kwargs(kwargs),
             )
         elif kwargs.get("package_name"):
-            _check_cli_arguments_none(kwargs, "python_file", "working_directory", "module_name")
+            _check_cli_arguments_none(kwargs, "python_file", "module_name")
             code_pointer = CodePointer.from_python_package(
                 kwargs.get("package_name"),
                 kwargs.get("attribute"),
+                get_working_directory_from_kwargs(kwargs),
             )
         else:
             check.failed("Must specify a Python file or module name")
-        return RepositoryPythonOrigin(executable_path=sys.executable, code_pointer=code_pointer)
+        return RepositoryPythonOrigin(
+            executable_path=sys.executable,
+            code_pointer=code_pointer,
+            entry_point=DEFAULT_DAGSTER_ENTRY_POINT,
+        )
 
     code_pointer_dict = _get_code_pointer_dict_from_kwargs(kwargs)
     if provided_repo_name is None and len(code_pointer_dict) == 1:
@@ -554,7 +583,11 @@ def get_repository_python_origin_from_kwargs(kwargs):
     else:
         code_pointer = code_pointer_dict[provided_repo_name]
 
-    return RepositoryPythonOrigin(executable_path=sys.executable, code_pointer=code_pointer)
+    return RepositoryPythonOrigin(
+        executable_path=sys.executable,
+        code_pointer=code_pointer,
+        entry_point=DEFAULT_DAGSTER_ENTRY_POINT,
+    )
 
 
 @contextmanager
@@ -652,7 +685,14 @@ def get_external_pipeline_or_job_from_external_repo(
     check.inst_param(external_repo, "external_repo", ExternalRepository)
     check.opt_str_param(provided_pipeline_or_job_name, "provided_pipeline_or_job_name")
 
-    external_pipelines = {ep.name: ep for ep in external_repo.get_all_external_pipelines()}
+    external_pipelines = {
+        ep.name: ep
+        for ep in (
+            external_repo.get_external_jobs()
+            if using_job_op_graph_apis
+            else external_repo.get_all_external_pipelines()
+        )
+    }
 
     check.invariant(external_pipelines)
 
