@@ -1,11 +1,14 @@
+import copy
 import inspect
 from abc import ABC, abstractmethod
-from datetime import datetime, time
+from datetime import datetime, time, timedelta
 from enum import Enum
 from typing import Any, Callable, Dict, Generic, List, NamedTuple, Optional, TypeVar, Union, cast
 
 import pendulum
 from dagster import check
+from dagster.serdes import whitelist_for_serdes
+from dateutil.relativedelta import relativedelta
 
 from ...seven.compat.pendulum import PendulumDateTime, to_timezone
 from ...utils import frozenlist, merge_dicts
@@ -15,6 +18,7 @@ from ..errors import (
     DagsterInvalidDefinitionError,
     DagsterInvalidInvocationError,
     DagsterInvariantViolationError,
+    DagsterUnknownPartitionError,
     ScheduleExecutionError,
     user_code_error_boundary,
 )
@@ -22,7 +26,11 @@ from ..storage.pipeline_run import PipelineRun
 from ..storage.tags import check_tags
 from .mode import DEFAULT_MODE_NAME
 from .run_request import RunRequest, SkipReason
-from .schedule import ScheduleDefinition, ScheduleEvaluationContext
+from .schedule_definition import (
+    DefaultScheduleStatus,
+    ScheduleDefinition,
+    ScheduleEvaluationContext,
+)
 from .utils import check_valid_name
 
 DEFAULT_DATE_FORMAT = "%Y-%m-%d"
@@ -32,7 +40,9 @@ T = TypeVar("T")
 
 class Partition(Generic[T]):
     """
-    Partition is the representation of a logical slice across an axis of a pipeline's work
+    A Partition represents a single slice of the entire set of a job's possible work. It consists
+    of a value, which is an object that represents that partition, and an optional name, which is
+    used to label the partition in a human-readable way.
 
     Args:
         value (Any): The object for this partition
@@ -51,6 +61,11 @@ class Partition(Generic[T]):
     def name(self) -> str:
         return self._name
 
+    def __eq__(self, other) -> bool:
+        return (
+            isinstance(other, Partition) and self.value == other.value and self.name == other.name
+        )
+
 
 def schedule_partition_range(
     start: datetime,
@@ -119,28 +134,82 @@ def schedule_partition_range(
     return partitions
 
 
+@whitelist_for_serdes
 class ScheduleType(Enum):
     HOURLY = "HOURLY"
     DAILY = "DAILY"
     WEEKLY = "WEEKLY"
     MONTHLY = "MONTHLY"
 
+    @property
+    def ordinal(self):
+        return {"HOURLY": 1, "DAILY": 2, "WEEKLY": 3, "MONTHLY": 4}[self.value]
+
+    @property
+    def delta(self):
+        if self == ScheduleType.HOURLY:
+            return timedelta(hours=1)
+        elif self == ScheduleType.DAILY:
+            return timedelta(days=1)
+        elif self == ScheduleType.WEEKLY:
+            return timedelta(weeks=1)
+        elif self == ScheduleType.MONTHLY:
+            return relativedelta(months=1)
+        else:
+            check.failed(f"Unexpected ScheduleType {self}")
+
+    def __gt__(self, other):
+        return self.ordinal > other.ordinal
+
+    def __lt__(self, other):
+        return self.ordinal < other.ordinal
+
 
 class PartitionsDefinition(ABC, Generic[T]):
     @abstractmethod
     def get_partitions(self, current_time: Optional[datetime] = None) -> List[Partition[T]]:
         ...
 
+    def __str__(self) -> str:
+        joined_keys = ", ".join([f"'{key}'" for key in self.get_partition_keys()])
+        return joined_keys
 
-class StaticPartitionsDefinition(PartitionsDefinition[T]):  # pylint: disable=unsubscriptable-object
-    def __init__(self, partitions: List[Partition[T]]):
-        self._partitions = check.list_param(partitions, "partitions", of_type=Partition)
+    def get_partition_keys(self, current_time: Optional[datetime] = None) -> List[str]:
+        return [partition.name for partition in self.get_partitions(current_time)]
+
+    def get_default_partition_mapping(self):
+        from dagster.core.asset_defs.partition_mapping import IdentityPartitionMapping
+
+        return IdentityPartitionMapping()
+
+
+class StaticPartitionsDefinition(
+    PartitionsDefinition[str],
+):  # pylint: disable=unsubscriptable-object
+    def __init__(self, partition_keys: List[str]):
+        check.list_param(partition_keys, "partition_keys", of_type=str)
+
+        # Dagit selects partition ranges following the format '2022-01-13...2022-01-14'
+        # "..." is an invalid substring in partition keys
+        if any(["..." in partition_key for partition_key in partition_keys]):
+            raise DagsterInvalidDefinitionError("'...' is an invalid substring in a partition key")
+
+        self._partitions = [Partition(key) for key in partition_keys]
 
     def get_partitions(
         self, current_time: Optional[datetime] = None  # pylint: disable=unused-argument
-    ) -> List[Partition[T]]:
+    ) -> List[Partition[str]]:
         return self._partitions
 
+    def __eq__(self, other) -> bool:
+        return (
+            isinstance(other, StaticPartitionsDefinition)
+            and self._partitions == other.get_partitions()
+        )
+
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}(partition_keys={[p.name for p in self._partitions]})"
+
 
 class ScheduleTimeBasedPartitionsDefinition(
     PartitionsDefinition[datetime],  # pylint: disable=unsubscriptable-object
@@ -257,16 +326,22 @@ class DynamicPartitionsDefinition(
     PartitionsDefinition,
     NamedTuple(
         "_DynamicPartitionsDefinition",
-        [("partition_fn", Callable[[Optional[datetime]], List[Partition]])],
+        [("partition_fn", Callable[[Optional[datetime]], Union[List[Partition], List[str]]])],
     ),
 ):
-    def __new__(cls, partition_fn: Callable[[Optional[datetime]], List[Partition]]):
+    def __new__(
+        cls, partition_fn: Callable[[Optional[datetime]], Union[List[Partition], List[str]]]
+    ):
         return super(DynamicPartitionsDefinition, cls).__new__(
             cls, check.callable_param(partition_fn, "partition_fn")
         )
 
     def get_partitions(self, current_time: Optional[datetime] = None) -> List[Partition]:
-        return self.partition_fn(current_time)
+        partitions = self.partition_fn(current_time)
+        if all(isinstance(partition, Partition) for partition in partitions):
+            return cast(List[Partition], partitions)
+        else:
+            return [Partition(p) for p in partitions]
 
 
 class PartitionSetDefinition(Generic[T]):
@@ -294,7 +369,7 @@ class PartitionSetDefinition(Generic[T]):
     def __init__(
         self,
         name: str,
-        pipeline_name: str,
+        pipeline_name: Optional[str] = None,
         partition_fn: Optional[Callable[..., Union[List[Partition[T]], List[str]]]] = None,
         solid_selection: Optional[List[str]] = None,
         mode: Optional[str] = None,
@@ -305,6 +380,7 @@ class PartitionSetDefinition(Generic[T]):
         partitions_def: Optional[
             PartitionsDefinition[T]  # pylint: disable=unsubscriptable-object
         ] = None,
+        job_name: Optional[str] = None,
     ):
         check.invariant(
             partition_fn is not None or partitions_def is not None,
@@ -314,6 +390,10 @@ class PartitionSetDefinition(Generic[T]):
             not (partition_fn and partitions_def),
             "Only one of `partition_fn` or `partitions_def` must be supplied.",
         )
+        check.invariant(
+            not (pipeline_name and job_name),
+            "Only one of `job_name` and `pipeline_name` must be supplied.",
+        )
 
         _wrap_partition_fn = None
 
@@ -347,6 +427,7 @@ class PartitionSetDefinition(Generic[T]):
 
         self._name = check_valid_name(name)
         self._pipeline_name = check.opt_str_param(pipeline_name, "pipeline_name")
+        self._job_name = check.opt_str_param(job_name, "job_name")
         self._partition_fn = _wrap_partition_fn
         self._solid_selection = check.opt_nullable_list_param(
             solid_selection, "solid_selection", of_type=str
@@ -358,14 +439,13 @@ class PartitionSetDefinition(Generic[T]):
         self._user_defined_tags_fn_for_partition = check.callable_param(
             tags_fn_for_partition, "tags_fn_for_partition"
         )
-        self._partitions_def = check.opt_inst_param(
-            partitions_def,
-            "partitions_def",
-            PartitionsDefinition,
-            default=DynamicPartitionsDefinition(partition_fn=_wrap_partition_fn)
-            if partition_fn is not None
-            else None,
-        )
+        check.opt_inst_param(partitions_def, "partitions_def", PartitionsDefinition)
+        if partitions_def is not None:
+            self._partitions_def = partitions_def
+        else:
+            if partition_fn is None:
+                check.failed("One of `partition_fn` or `partitions_def` must be supplied.")
+            self._partitions_def = DynamicPartitionsDefinition(partition_fn=_wrap_partition_fn)
 
     @property
     def name(self):
@@ -375,6 +455,10 @@ class PartitionSetDefinition(Generic[T]):
     def pipeline_name(self):
         return self._pipeline_name
 
+    @property
+    def job_name(self):
+        return self._job_name
+
     @property
     def solid_selection(self):
         return self._solid_selection
@@ -384,10 +468,10 @@ class PartitionSetDefinition(Generic[T]):
         return self._mode
 
     def run_config_for_partition(self, partition: Partition[T]) -> Dict[str, Any]:
-        return self._user_defined_run_config_fn_for_partition(partition)
+        return copy.deepcopy(self._user_defined_run_config_fn_for_partition(partition))
 
     def tags_for_partition(self, partition: Partition[T]) -> Dict[str, str]:
-        user_tags = self._user_defined_tags_fn_for_partition(partition)
+        user_tags = copy.deepcopy(self._user_defined_tags_fn_for_partition(partition))
         check_tags(user_tags, "user_tags")
 
         tags = merge_dicts(user_tags, PipelineRun.tags_for_partition_set(self, partition))
@@ -425,6 +509,7 @@ class PartitionSetDefinition(Generic[T]):
         description=None,
         decorated_fn=None,
         job=None,
+        default_status=DefaultScheduleStatus.STOPPED,
     ):
         """Create a ScheduleDefinition from a PartitionSetDefinition.
 
@@ -440,9 +525,12 @@ class PartitionSetDefinition(Generic[T]):
                 determines whether a schedule should execute. Defaults to a function that always returns
                 ``True``.
             environment_vars (Optional[dict]): The environment variables to set for the schedule.
-            execution_timezone (Optional[str]): Timezone in which the schedule should run. Only works
-                with DagsterDaemonScheduler, and must be set when using that scheduler.
+            execution_timezone (Optional[str]): Timezone in which the schedule should run.
+                Supported strings for timezones are the ones provided by the
+                `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
             description (Optional[str]): A human-readable description of the schedule.
+            default_status (DefaultScheduleStatus): Whether the schedule starts as running or not. The default
+                status can be overridden from Dagit or via the GraphQL API.
 
         Returns:
             PartitionScheduleDefinition: The generated PartitionScheduleDefinition for the partition
@@ -456,6 +544,7 @@ class PartitionSetDefinition(Generic[T]):
         check.callable_param(partition_selector, "partition_selector")
         check.opt_str_param(execution_timezone, "execution_timezone")
         check.opt_str_param(description, "description")
+        check.inst_param(default_status, "default_status", DefaultScheduleStatus)
 
         def _execution_fn(context):
             check.inst_param(context, "context", ScheduleEvaluationContext)
@@ -540,6 +629,7 @@ class PartitionSetDefinition(Generic[T]):
             description=description,
             decorated_fn=decorated_fn,
             job=job,
+            default_status=default_status,
         )
 
 
@@ -563,6 +653,7 @@ class PartitionScheduleDefinition(ScheduleDefinition):
         description=None,
         decorated_fn=None,
         job=None,
+        default_status=DefaultScheduleStatus.STOPPED,
     ):
         super(PartitionScheduleDefinition, self).__init__(
             name=check_valid_name(name),
@@ -578,6 +669,7 @@ class PartitionScheduleDefinition(ScheduleDefinition):
             execution_fn=execution_fn,
             description=description,
             job=job,
+            default_status=default_status,
         )
         self._partition_set = check.inst_param(
             partition_set, "partition_set", PartitionSetDefinition
@@ -620,17 +712,23 @@ class PartitionScheduleDefinition(ScheduleDefinition):
 
 class PartitionedConfig(Generic[T]):
     """Defines a way of configuring a job where the job can be run on one of a discrete set of
-    partitions, and each partition corresponds to run configuration for the job."""
+    partitions, and each partition corresponds to run configuration for the job.
+
+    Setting PartitionedConfig as the config for a job allows you to launch backfills for that job
+    and view the run history across partitions.
+    """
 
     def __init__(
         self,
         partitions_def: PartitionsDefinition[T],  # pylint: disable=unsubscriptable-object
         run_config_for_partition_fn: Callable[[Partition[T]], Dict[str, Any]],
+        decorated_fn: Optional[Callable[..., Dict[str, Any]]] = None,
     ):
         self._partitions = check.inst_param(partitions_def, "partitions_def", PartitionsDefinition)
         self._run_config_for_partition_fn = check.callable_param(
             run_config_for_partition_fn, "run_config_for_partition_fn"
         )
+        self._decorated_fn = decorated_fn
 
     @property
     def partitions_def(self) -> PartitionsDefinition[T]:  # pylint: disable=unsubscriptable-object
@@ -640,23 +738,121 @@ class PartitionedConfig(Generic[T]):
     def run_config_for_partition_fn(self) -> Callable[[Partition[T]], Dict[str, Any]]:
         return self._run_config_for_partition_fn
 
+    def get_partition_keys(self, current_time: Optional[datetime] = None) -> List[str]:
+        return [partition.name for partition in self.partitions_def.get_partitions(current_time)]
+
+    def get_run_config(self, partition_key: str) -> Dict[str, Any]:
+        matching = [
+            partition
+            for partition in self.partitions_def.get_partitions()
+            if partition.name == partition_key
+        ]
+        if not matching:
+            raise DagsterUnknownPartitionError(
+                f"Could not find a partition with key `{partition_key}`"
+            )
+        return self.run_config_for_partition_fn(matching[0])
+
+    def __call__(self, *args, **kwargs):
+        if self._decorated_fn is None:
+            raise DagsterInvalidInvocationError(
+                "Only PartitionedConfig objects created using one of the partitioned config "
+                "decorators can be directly invoked."
+            )
+        else:
+            return self._decorated_fn(*args, **kwargs)
+
+
+def static_partitioned_config(
+    partition_keys: List[str],
+) -> Callable[[Callable[[str], Dict[str, Any]]], PartitionedConfig]:
+    """Creates a static partitioned config for a job.
+
+    The provided partition_keys returns a static list of strings identifying the set of partitions,
+    given an optional datetime argument (representing the current time).  The list of partitions
+    is static, so while the run config returned by the decorated function may change over time, the
+    list of valid partition keys does not.
+
+    This has performance advantages over `dynamic_partitioned_config` in terms of loading different
+    partition views in Dagit.
+
+    The decorated function takes in a partition key and returns a valid run config for a particular
+    target job.
+
+    Args:
+        partition_keys (List[str]): A list of valid partition keys, which serve as the range of
+            values that can be provided to the decorated run config function.
+
+    Returns:
+        PartitionedConfig
+    """
+    check.list_param(partition_keys, "partition_keys", str)
+
+    def inner(fn: Callable[[str], Dict[str, Any]]) -> PartitionedConfig:
+        check.callable_param(fn, "fn")
+
+        def _run_config_wrapper(partition: Partition[T]) -> Dict[str, Any]:
+            return fn(partition.name)
+
+        return PartitionedConfig(
+            partitions_def=StaticPartitionsDefinition(partition_keys),
+            run_config_for_partition_fn=_run_config_wrapper,
+            decorated_fn=fn,
+        )
+
+    return inner
+
+
+def dynamic_partitioned_config(
+    partition_fn: Callable[[Optional[datetime]], List[str]],
+) -> Callable[[Callable[[str], Dict[str, Any]]], PartitionedConfig]:
+    """Creates a dynamic partitioned config for a job.
+
+    The provided partition_fn returns a list of strings identifying the set of partitions, given
+    an optional datetime argument (representing the current time).  The list of partitions returned
+    may change over time.
+
+    The decorated function takes in a partition key and returns a valid run config for a particular
+    target job.
+
+    Args:
+        partition_fn (Callable[[datetime.datetime], Sequence[str]]): A function that generates a
+            list of valid partition keys, which serve as the range of values that can be provided
+            to the decorated run config function.
+
+    Returns:
+        PartitionedConfig
+    """
+    check.callable_param(partition_fn, "partition_fn")
+
+    def inner(fn: Callable[[str], Dict[str, Any]]) -> PartitionedConfig:
+        def _run_config_wrapper(partition: Partition[T]) -> Dict[str, Any]:
+            return fn(partition.name)
+
+        return PartitionedConfig(
+            partitions_def=DynamicPartitionsDefinition(partition_fn),
+            run_config_for_partition_fn=_run_config_wrapper,
+            decorated_fn=fn,
+        )
+
+    return inner
+
 
 def get_cron_schedule(
     schedule_type: ScheduleType,
     time_of_day: time = time(0, 0),
-    day_of_week: Optional[int] = 0,
+    execution_day: Optional[int] = None,
 ) -> str:
     minute = time_of_day.minute
     hour = time_of_day.hour
-    day = day_of_week
 
     if schedule_type is ScheduleType.HOURLY:
         return f"{minute} * * * *"
     elif schedule_type is ScheduleType.DAILY:
         return f"{minute} {hour} * * *"
     elif schedule_type is ScheduleType.WEEKLY:
-        return f"{minute} {hour} * * {day}"
+        return f"{minute} {hour} * * {execution_day if execution_day != None else 0}"
     elif schedule_type is ScheduleType.MONTHLY:
-        return f"{minute} {hour} {day} * *"
+        return f"{minute} {hour} {execution_day if execution_day != None else 1} * *"
     else:
         check.assert_never(schedule_type)
