@@ -8,10 +8,11 @@ from dagster import check
 from dagster.core.definitions import (
     AssetKey,
     AssetMaterialization,
-    EventMetadataEntry,
+    AssetObservation,
     ExpectationResult,
     HookDefinition,
     Materialization,
+    MetadataEntry,
     NodeHandle,
 )
 from dagster.core.definitions.events import AssetLineageInfo, ObjectStoreOperationType
@@ -59,6 +60,7 @@ if TYPE_CHECKING:
         "HandledOutputData",
         "LoadedInputData",
         "ComputeLogsCaptureData",
+        "AssetObservationData",
     ]
 
 
@@ -76,6 +78,7 @@ class DagsterEventType(Enum):
     STEP_RESTARTED = "STEP_RESTARTED"
 
     ASSET_MATERIALIZATION = "ASSET_MATERIALIZATION"
+    ASSET_OBSERVATION = "ASSET_OBSERVATION"
     STEP_EXPECTATION_RESULT = "STEP_EXPECTATION_RESULT"
 
     # We want to display RUN_* events in dagit and in our LogManager output, but in order to
@@ -137,6 +140,7 @@ STEP_EVENTS = {
     DagsterEventType.STEP_SUCCESS,
     DagsterEventType.STEP_SKIPPED,
     DagsterEventType.ASSET_MATERIALIZATION,
+    DagsterEventType.ASSET_OBSERVATION,
     DagsterEventType.STEP_EXPECTATION_RESULT,
     DagsterEventType.OBJECT_STORE_OPERATION,
     DagsterEventType.HANDLED_OUTPUT,
@@ -186,6 +190,11 @@ EVENT_TYPE_TO_PIPELINE_RUN_STATUS = {
 
 PIPELINE_RUN_STATUS_TO_EVENT_TYPE = {v: k for k, v in EVENT_TYPE_TO_PIPELINE_RUN_STATUS.items()}
 
+ASSET_EVENTS = {
+    DagsterEventType.ASSET_MATERIALIZATION,
+    DagsterEventType.ASSET_OBSERVATION,
+}
+
 
 def _assert_type(
     method: str, expected_type: DagsterEventType, actual_type: DagsterEventType
@@ -490,17 +499,31 @@ class DagsterEvent(
     def is_step_materialization(self) -> bool:
         return self.event_type == DagsterEventType.ASSET_MATERIALIZATION
 
+    @property
+    def is_expectation_result(self) -> bool:
+        return self.event_type == DagsterEventType.STEP_EXPECTATION_RESULT
+
+    @property
+    def is_asset_observation(self) -> bool:
+        return self.event_type == DagsterEventType.ASSET_OBSERVATION
+
     @property
     def asset_key(self) -> Optional[AssetKey]:
-        if self.event_type != DagsterEventType.ASSET_MATERIALIZATION:
+        if self.event_type == DagsterEventType.ASSET_MATERIALIZATION:
+            return self.step_materialization_data.materialization.asset_key
+        elif self.event_type == DagsterEventType.ASSET_OBSERVATION:
+            return self.asset_observation_data.asset_observation.asset_key
+        else:
             return None
-        return self.step_materialization_data.materialization.asset_key
 
     @property
     def partition(self) -> Optional[str]:
-        if self.event_type != DagsterEventType.ASSET_MATERIALIZATION:
+        if self.event_type == DagsterEventType.ASSET_MATERIALIZATION:
+            return self.step_materialization_data.materialization.partition
+        elif self.event_type == DagsterEventType.ASSET_OBSERVATION:
+            return self.asset_observation_data.asset_observation.partition
+        else:
             return None
-        return self.step_materialization_data.materialization.partition
 
     @property
     def step_input_data(self) -> "StepInputData":
@@ -542,6 +565,11 @@ class DagsterEvent(
         )
         return cast(StepMaterializationData, self.event_specific_data)
 
+    @property
+    def asset_observation_data(self) -> "AssetObservationData":
+        _assert_type("asset_observation_data", DagsterEventType.ASSET_OBSERVATION, self.event_type)
+        return cast(AssetObservationData, self.event_specific_data)
+
     @property
     def step_expectation_result_data(self) -> "StepExpectationResultData":
         _assert_type(
@@ -724,6 +752,16 @@ class DagsterEvent(
             ),
         )
 
+    @staticmethod
+    def asset_observation(
+        step_context: IStepContext, observation: AssetObservation
+    ) -> "DagsterEvent":
+        return DagsterEvent.from_step(
+            event_type=DagsterEventType.ASSET_OBSERVATION,
+            step_context=step_context,
+            event_specific_data=AssetObservationData(observation),
+        )
+
     @staticmethod
     def step_expectation_result(
         step_context: IStepContext, expectation_result: ExpectationResult
@@ -844,7 +882,7 @@ class DagsterEvent(
             resource_obj = resource_instances[resource_key]
             resource_time = resource_init_times[resource_key]
             metadata_entries.append(
-                EventMetadataEntry.python_artifact(
+                MetadataEntry.python_artifact(
                     resource_obj.__class__, resource_key, "Initialized in {}".format(resource_time)
                 )
             )
@@ -989,7 +1027,7 @@ class DagsterEvent(
                 value_name=value_name,
                 address=object_store_operation_result.key,
                 metadata_entries=[
-                    EventMetadataEntry.path(object_store_operation_result.key, label="key")
+                    MetadataEntry.path(object_store_operation_result.key, label="key")
                 ],
                 version=object_store_operation_result.version,
                 mapping_key=object_store_operation_result.mapping_key,
@@ -1003,7 +1041,7 @@ class DagsterEvent(
         output_name: str,
         manager_key: str,
         message_override: Optional[str] = None,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
     ) -> "DagsterEvent":
         message = f'Handled output "{output_name}" using IO manager "{manager_key}"'
         return DagsterEvent.from_step(
@@ -1121,7 +1159,7 @@ class DagsterEvent(
     def capture_logs(pipeline_context: IPlanContext, log_key: str, steps: List["ExecutionStep"]):
         step_keys = [step.key for step in steps]
         if len(step_keys) == 1:
-            message = f"Started capturing logs for solid: {step_keys[0]}."
+            message = f"Started capturing logs for step: {step_keys[0]}."
         else:
             message = f"Started capturing logs in process (pid: {os.getpid()})."
 
@@ -1163,6 +1201,19 @@ def get_step_output_event(
     return None
 
 
+@whitelist_for_serdes
+class AssetObservationData(
+    NamedTuple("_AssetObservation", [("asset_observation", AssetObservation)])
+):
+    def __new__(cls, asset_observation: AssetObservation):
+        return super(AssetObservationData, cls).__new__(
+            cls,
+            asset_observation=check.inst_param(
+                asset_observation, "asset_observation", AssetObservation
+            ),
+        )
+
+
 @whitelist_for_serdes
 class StepMaterializationData(
     NamedTuple(
@@ -1214,7 +1265,7 @@ class ObjectStoreOperationResultData(
         [
             ("op", ObjectStoreOperationType),
             ("value_name", Optional[str]),
-            ("metadata_entries", List[EventMetadataEntry]),
+            ("metadata_entries", List[MetadataEntry]),
             ("address", Optional[str]),
             ("version", Optional[str]),
             ("mapping_key", Optional[str]),
@@ -1225,7 +1276,7 @@ class ObjectStoreOperationResultData(
         cls,
         op: ObjectStoreOperationType,
         value_name: Optional[str] = None,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
         address: Optional[str] = None,
         version: Optional[str] = None,
         mapping_key: Optional[str] = None,
@@ -1235,7 +1286,7 @@ class ObjectStoreOperationResultData(
             op=cast(ObjectStoreOperationType, check.str_param(op, "op")),
             value_name=check.opt_str_param(value_name, "value_name"),
             metadata_entries=check.opt_list_param(
-                metadata_entries, "metadata_entries", of_type=EventMetadataEntry
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
             address=check.opt_str_param(address, "address"),
             version=check.opt_str_param(version, "version"),
@@ -1248,7 +1299,7 @@ class EngineEventData(
     NamedTuple(
         "_EngineEventData",
         [
-            ("metadata_entries", List[EventMetadataEntry]),
+            ("metadata_entries", List[MetadataEntry]),
             ("error", Optional[SerializableErrorInfo]),
             ("marker_start", Optional[str]),
             ("marker_end", Optional[str]),
@@ -1261,7 +1312,7 @@ class EngineEventData(
     #
     def __new__(
         cls,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
         error: Optional[SerializableErrorInfo] = None,
         marker_start: Optional[str] = None,
         marker_end: Optional[str] = None,
@@ -1269,7 +1320,7 @@ class EngineEventData(
         return super(EngineEventData, cls).__new__(
             cls,
             metadata_entries=check.opt_list_param(
-                metadata_entries, "metadata_entries", of_type=EventMetadataEntry
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
             error=check.opt_inst_param(error, "error", SerializableErrorInfo),
             marker_start=check.opt_str_param(marker_start, "marker_start"),
@@ -1281,9 +1332,9 @@ class EngineEventData(
         pid: int, step_keys_to_execute: Optional[List[str]] = None, marker_end: Optional[str] = None
     ) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[EventMetadataEntry.text(str(pid), "pid")]
+            metadata_entries=[MetadataEntry.text(str(pid), "pid")]
             + (
-                [EventMetadataEntry.text(str(step_keys_to_execute), "step_keys")]
+                [MetadataEntry.text(str(step_keys_to_execute), "step_keys")]
                 if step_keys_to_execute
                 else []
             ),
@@ -1295,9 +1346,9 @@ class EngineEventData(
         pid: int, step_keys_to_execute: Optional[List[str]] = None
     ) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[EventMetadataEntry.text(str(pid), "pid")]
+            metadata_entries=[MetadataEntry.text(str(pid), "pid")]
             + (
-                [EventMetadataEntry.text(str(step_keys_to_execute), "step_keys")]
+                [MetadataEntry.text(str(step_keys_to_execute), "step_keys")]
                 if step_keys_to_execute
                 else []
             )
@@ -1306,7 +1357,7 @@ class EngineEventData(
     @staticmethod
     def interrupted(steps_interrupted: List[str]) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[EventMetadataEntry.text(str(steps_interrupted), "steps_interrupted")]
+            metadata_entries=[MetadataEntry.text(str(steps_interrupted), "steps_interrupted")]
         )
 
     @staticmethod
@@ -1366,7 +1417,7 @@ class HandledOutputData(
         [
             ("output_name", str),
             ("manager_key", str),
-            ("metadata_entries", List[EventMetadataEntry]),
+            ("metadata_entries", List[MetadataEntry]),
         ],
     )
 ):
@@ -1374,14 +1425,14 @@ class HandledOutputData(
         cls,
         output_name: str,
         manager_key: str,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
     ):
         return super(HandledOutputData, cls).__new__(
             cls,
             output_name=check.str_param(output_name, "output_name"),
             manager_key=check.str_param(manager_key, "manager_key"),
             metadata_entries=check.opt_list_param(
-                metadata_entries, "metadata_entries", of_type=EventMetadataEntry
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
         )
 
