@@ -1,3 +1,4 @@
+import inspect
 from collections import defaultdict
 from typing import Any, Dict, Iterator, List, Optional, Set, Tuple, Union, cast
 
@@ -5,6 +6,7 @@ from dagster import check
 from dagster.core.definitions import (
     AssetKey,
     AssetMaterialization,
+    AssetObservation,
     ExpectationResult,
     Materialization,
     Output,
@@ -13,8 +15,12 @@ from dagster.core.definitions import (
     TypeCheck,
 )
 from dagster.core.definitions.decorators.solid import DecoratedSolidFunction
-from dagster.core.definitions.event_metadata import EventMetadataEntry, PartitionMetadataEntry
 from dagster.core.definitions.events import AssetLineageInfo, DynamicOutput
+from dagster.core.definitions.metadata import (
+    MetadataEntry,
+    PartitionMetadataEntry,
+    normalize_metadata,
+)
 from dagster.core.errors import (
     DagsterExecutionHandleOutputError,
     DagsterInvariantViolationError,
@@ -59,8 +65,6 @@ def _step_output_error_checked_user_event_sequence(
     step = step_context.step
     op_label = step_context.describe_op()
     output_names = list([output_def.name for output_def in step.step_outputs])
-    seen_outputs: Set[str] = set()
-    seen_mapping_keys: Dict[str, Set[str]] = defaultdict(set)
 
     for user_event in user_event_sequence:
         if not isinstance(user_event, (Output, DynamicOutput)):
@@ -81,7 +85,7 @@ def _step_output_error_checked_user_event_sequence(
         )
 
         if isinstance(output, Output):
-            if output.output_name in seen_outputs:
+            if step_context.has_seen_output(output.output_name):
                 raise DagsterInvariantViolationError(
                     f'Compute for {op_label} returned an output "{output.output_name}" multiple '
                     "times"
@@ -92,25 +96,44 @@ def _step_output_error_checked_user_event_sequence(
                     f'Compute for {op_label} for output "{output.output_name}" defined as dynamic '
                     "must yield DynamicOutput, got Output."
                 )
+
+            step_context.observe_output(output.output_name)
+
+            metadata = step_context.get_output_metadata(output.output_name)
+            output = Output(
+                value=output.value,
+                output_name=output.output_name,
+                metadata_entries=output.metadata_entries
+                + normalize_metadata(cast(Dict[str, Any], metadata), []),
+            )
         else:
             if not output_def.is_dynamic:
                 raise DagsterInvariantViolationError(
                     f"Compute for {op_label} yielded a DynamicOutput, but did not use "
                     "DynamicOutputDefinition."
                 )
-            if output.mapping_key in seen_mapping_keys[output.output_name]:
+            if step_context.has_seen_output(output.output_name, output.mapping_key):
                 raise DagsterInvariantViolationError(
                     f"Compute for {op_label} yielded a DynamicOutput with mapping_key "
                     f'"{output.mapping_key}" multiple times.'
                 )
-            seen_mapping_keys[output.output_name].add(output.mapping_key)
+            step_context.observe_output(output.output_name, output.mapping_key)
+            metadata = step_context.get_output_metadata(
+                output.output_name, mapping_key=output.mapping_key
+            )
+            output = DynamicOutput(
+                value=output.value,
+                output_name=output.output_name,
+                metadata_entries=output.metadata_entries
+                + normalize_metadata(cast(Dict[str, Any], metadata), []),
+                mapping_key=output.mapping_key,
+            )
 
         yield output
-        seen_outputs.add(output.output_name)
 
     for step_output in step.step_outputs:
         step_output_def = step_context.solid_def.output_def_named(step_output.name)
-        if not step_output_def.name in seen_outputs and not step_output_def.optional:
+        if not step_context.has_seen_output(step_output_def.name) and not step_output_def.optional:
             if step_output_def.dagster_type.kind == DagsterTypeKind.NOTHING:
                 step_context.log.info(
                     f'Emitting implicit Nothing for output "{step_output_def.name}" on {op_label}'
@@ -239,16 +262,17 @@ def _type_check_output(
             ),
             version=version,
             metadata_entries=[
-                entry for entry in output.metadata_entries if isinstance(entry, EventMetadataEntry)
+                entry for entry in output.metadata_entries if isinstance(entry, MetadataEntry)
             ],
         ),
     )
 
     if not type_check.success:
         raise DagsterTypeCheckDidNotPass(
-            description='Type check failed for step output "{output_name}" - expected type "{dagster_type}".'.format(
-                output_name=output.output_name,
-                dagster_type=dagster_type.display_name,
+            description=(
+                f'Type check failed for step output "{output.output_name}" - '
+                f'expected type "{dagster_type.display_name}". '
+                f"Description: {type_check.description}"
             ),
             metadata_entries=type_check.metadata_entries,
             dagster_type=dagster_type,
@@ -272,7 +296,6 @@ def core_dagster_event_sequence_for_step(
         yield DagsterEvent.step_start_event(step_context)
 
     inputs = {}
-    input_lineage = []
 
     for step_input in step_context.step.step_inputs:
         input_def = step_input.source.get_input_def(step_context.pipeline_def)
@@ -280,9 +303,6 @@ def core_dagster_event_sequence_for_step(
 
         if dagster_type.kind == DagsterTypeKind.NOTHING:
             continue
-
-        input_lineage.extend(step_input.source.get_asset_lineage(step_context))
-
         for event_or_input_value in ensure_gen(step_input.source.load_input_object(step_context)):
             if isinstance(event_or_input_value, DagsterEvent):
                 yield event_or_input_value
@@ -296,7 +316,7 @@ def core_dagster_event_sequence_for_step(
         ):
             yield evt
 
-    input_lineage = _dedup_asset_lineage(input_lineage)
+    input_lineage = step_context.get_input_lineage()
 
     # The core execution loop expects a compute generator in a specific format: a generator that
     # takes a context and dictionary of inputs as input, yields output events. If a solid definition
@@ -322,14 +342,17 @@ def core_dagster_event_sequence_for_step(
         for user_event in check.generator(
             _step_output_error_checked_user_event_sequence(step_context, user_event_sequence)
         ):
-
-            if isinstance(user_event, (Output, DynamicOutput)):
+            if isinstance(user_event, DagsterEvent):
+                yield user_event
+            elif isinstance(user_event, (Output, DynamicOutput)):
                 for evt in _type_check_and_store_output(step_context, user_event, input_lineage):
                     yield evt
             # for now, I'm ignoring AssetMaterializations yielded manually, but we might want
             # to do something with these in the above path eventually
             elif isinstance(user_event, (AssetMaterialization, Materialization)):
                 yield DagsterEvent.asset_materialization(step_context, user_event, input_lineage)
+            elif isinstance(user_event, AssetObservation):
+                yield DagsterEvent.asset_observation(step_context, user_event)
             elif isinstance(user_event, ExpectationResult):
                 yield DagsterEvent.step_expectation_result(step_context, user_event)
             else:
@@ -364,8 +387,9 @@ def _type_check_and_store_output(
     # to be directly captured to a dictionary after they are computed.
     if step_context.output_capture is not None:
         step_context.output_capture[step_output_handle] = output.value
-    # capture output at the step level for threading th computed output values to hook context
-    step_context.step_output_capture[step_output_handle] = output.value
+    # capture output at the step level for threading the computed output values to hook context
+    if step_context.step_output_capture is not None:
+        step_context.step_output_capture[step_output_handle] = output.value
 
     version = (
         resolve_step_output_versions(
@@ -436,13 +460,13 @@ def _get_output_asset_materializations(
     asset_partitions: Set[str],
     output: Union[Output, DynamicOutput],
     output_def: OutputDefinition,
-    io_manager_metadata_entries: List[Union[EventMetadataEntry, PartitionMetadataEntry]],
+    io_manager_metadata_entries: List[Union[MetadataEntry, PartitionMetadataEntry]],
 ) -> Iterator[AssetMaterialization]:
 
     all_metadata = output.metadata_entries + io_manager_metadata_entries
 
     if asset_partitions:
-        metadata_mapping: Dict[str, List["EventMetadataEntry"]] = {
+        metadata_mapping: Dict[str, List[Union[MetadataEntry, PartitionMetadataEntry]]] = {
             partition: [] for partition in asset_partitions
         }
         for entry in all_metadata:
@@ -472,9 +496,7 @@ def _get_output_asset_materializations(
                     f"Output {output_def.name} got a PartitionMetadataEntry ({entry}), but "
                     "is not associated with any specific partitions."
                 )
-        yield AssetMaterialization(
-            asset_key=asset_key, metadata_entries=cast(List["EventMetadataEntry"], all_metadata)
-        )
+        yield AssetMaterialization(asset_key=asset_key, metadata_entries=all_metadata)
 
 
 def _store_output(
@@ -488,40 +510,79 @@ def _store_output(
     output_manager = step_context.get_io_manager(step_output_handle)
     output_context = step_context.get_output_context(step_output_handle)
 
-    handle_output_res = output_manager.handle_output(output_context, output.value)
-
     manager_materializations = []
-    manager_metadata_entries = []
-    if handle_output_res is not None:
-        for elt in iterate_with_context(
-            lambda: solid_execution_error_boundary(
-                DagsterExecutionHandleOutputError,
-                msg_fn=lambda: (
-                    f'Error occurred while handling output "{output_context.name}" of '
-                    f'step "{step_context.step.key}":'
-                ),
-                step_context=step_context,
-                step_key=step_context.step.key,
-                output_name=output_context.name,
-            ),
-            ensure_gen(handle_output_res),
-        ):
-            if isinstance(elt, AssetMaterialization):
-                manager_materializations.append(elt)
-            elif isinstance(elt, (EventMetadataEntry, PartitionMetadataEntry)):
-                experimental_functionality_warning(
-                    "Yielding metadata from an IOManager's handle_output() function"
-                )
-                manager_metadata_entries.append(elt)
-            else:
-                raise DagsterInvariantViolationError(
-                    f"IO manager on output {output_def.name} has returned "
-                    f"value {elt} of type {type(elt).__name__}. The return type can only be "
-                    "one of AssetMaterialization, EventMetadataEntry, PartitionMetadataEntry."
-                )
+    manager_metadata_entries: List[Union[PartitionMetadataEntry, MetadataEntry]] = []
 
+    # output_manager.handle_output is either a generator function, or a normal function with or
+    # without a return value. In the case that handle_output is a normal function, we need to
+    # catch errors should they be raised before a return value. We can do this by wrapping
+    # handle_output in a generator so that errors will be caught within iterate_with_context.
+
+    if not inspect.isgeneratorfunction(output_manager.handle_output):
+
+        def _gen_fn():
+            gen_output = output_manager.handle_output(output_context, output.value)
+            for event in output_context.consume_events():
+                yield event
+            if gen_output:
+                yield gen_output
+
+        handle_output_gen = _gen_fn()
+    else:
+        handle_output_gen = output_manager.handle_output(output_context, output.value)
+
+    for elt in iterate_with_context(
+        lambda: solid_execution_error_boundary(
+            DagsterExecutionHandleOutputError,
+            msg_fn=lambda: (
+                f'Error occurred while handling output "{output_context.name}" of '
+                f'step "{step_context.step.key}":'
+            ),
+            step_context=step_context,
+            step_key=step_context.step.key,
+            output_name=output_context.name,
+        ),
+        handle_output_gen,
+    ):
+        for event in output_context.consume_events():
+            yield event
+
+        manager_metadata_entries.extend(output_context.consume_logged_metadata_entries())
+        if isinstance(elt, DagsterEvent):
+            yield elt
+        elif isinstance(elt, AssetMaterialization):
+            manager_materializations.append(elt)
+        elif isinstance(elt, (MetadataEntry, PartitionMetadataEntry)):
+            experimental_functionality_warning(
+                "Yielding metadata from an IOManager's handle_output() function"
+            )
+            manager_metadata_entries.append(elt)
+        else:
+            raise DagsterInvariantViolationError(
+                f"IO manager on output {output_def.name} has returned "
+                f"value {elt} of type {type(elt).__name__}. The return type can only be "
+                "one of AssetMaterialization, MetadataEntry, PartitionMetadataEntry."
+            )
+
+    for event in output_context.consume_events():
+        yield event
+
+    manager_metadata_entries.extend(output_context.consume_logged_metadata_entries())
     # do not alter explicitly created AssetMaterializations
     for materialization in manager_materializations:
+        if materialization.metadata_entries and manager_metadata_entries:
+            raise DagsterInvariantViolationError(
+                f"When handling output '{output_context.name}' of {output_context.solid_def.node_type_str} '{output_context.solid_def.name}', received a materialization with metadata, while context.add_output_metadata was used within the same call to handle_output. Due to potential conflicts, this is not allowed. Please specify metadata in one place within the `handle_output` function."
+            )
+        if manager_metadata_entries:
+            materialization = AssetMaterialization(
+                asset_key=materialization.asset_key,
+                description=materialization.description,
+                metadata_entries=manager_metadata_entries,
+                partition=materialization.partition,
+                tags=materialization.tags,
+                metadata=None,
+            )
         yield DagsterEvent.asset_materialization(step_context, materialization, input_lineage)
 
     asset_key, partitions = _asset_key_and_partitions_for_output(
@@ -542,7 +603,7 @@ def _store_output(
         output_name=step_output_handle.output_name,
         manager_key=output_def.io_manager_key,
         metadata_entries=[
-            entry for entry in manager_metadata_entries if isinstance(entry, EventMetadataEntry)
+            entry for entry in manager_metadata_entries if isinstance(entry, MetadataEntry)
         ],
     )
 
