@@ -1,4 +1,5 @@
 import math
+import multiprocessing
 import os
 import queue
 import sys
@@ -13,13 +14,12 @@ from time import sleep
 import grpc
 from dagster import check, seven
 from dagster.core.code_pointer import CodePointer
-from dagster.core.definitions.reconstructable import (
-    ReconstructableRepository,
-    repository_def_from_target_def,
-)
+from dagster.core.definitions.reconstructable import ReconstructableRepository
+from dagster.core.errors import DagsterUserCodeUnreachableError
 from dagster.core.host_representation.external_data import external_repository_data_from_def
 from dagster.core.host_representation.origin import ExternalPipelineOrigin, ExternalRepositoryOrigin
 from dagster.core.instance import DagsterInstance
+from dagster.core.origin import DEFAULT_DAGSTER_ENTRY_POINT, get_python_environment_entry_point
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster.serdes import (
     deserialize_json_to_dagster_namedtuple,
@@ -27,8 +27,7 @@ from dagster.serdes import (
     whitelist_for_serdes,
 )
 from dagster.serdes.ipc import IPCErrorMessage, ipc_write_stream, open_ipc_subprocess
-from dagster.seven import multiprocessing
-from dagster.utils import find_free_port, safe_tempfile_path_unmanaged
+from dagster.utils import find_free_port, frozenlist, safe_tempfile_path_unmanaged
 from dagster.utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 from grpc_health.v1 import health, health_pb2, health_pb2_grpc
 
@@ -67,7 +66,7 @@ from .types import (
     ShutdownServerResult,
     StartRunResult,
 )
-from .utils import get_loadable_targets
+from .utils import get_loadable_targets, max_rx_bytes, max_send_bytes
 
 EVENT_QUEUE_POLL_INTERVAL = 0.1
 
@@ -80,19 +79,44 @@ class CouldNotBindGrpcServerToAddress(Exception):
     pass
 
 
-class RepositorySymbolsAndCodePointers:
-    def __init__(self, loadable_target_origin):
+class LoadedRepositories:
+    def __init__(self, loadable_target_origin, entry_point):
         self._loadable_target_origin = loadable_target_origin
-        self._loadable_repository_symbols = None
-        self._code_pointers_by_repo_name = None
 
-    def load(self):
-        self._loadable_repository_symbols = load_loadable_repository_symbols(
-            self._loadable_target_origin
-        )
-        self._code_pointers_by_repo_name = build_code_pointers_by_repo_name(
-            self._loadable_target_origin, self._loadable_repository_symbols
+        self._code_pointers_by_repo_name = {}
+        self._recon_repos_by_name = {}
+        self._loadable_repository_symbols = []
+
+        if not loadable_target_origin:
+            return
+
+        loadable_targets = get_loadable_targets(
+            loadable_target_origin.python_file,
+            loadable_target_origin.module_name,
+            loadable_target_origin.package_name,
+            loadable_target_origin.working_directory,
+            loadable_target_origin.attribute,
         )
+        for loadable_target in loadable_targets:
+            pointer = _get_code_pointer(loadable_target_origin, loadable_target)
+            recon_repo = ReconstructableRepository(
+                pointer,
+                _get_current_image(),
+                sys.executable,
+                entry_point=entry_point,
+            )
+            repo_def = recon_repo.get_definition()
+            # force load of all lazy constructed jobs/pipelines
+            repo_def.get_all_pipelines()
+
+            self._code_pointers_by_repo_name[repo_def.name] = pointer
+            self._recon_repos_by_name[repo_def.name] = recon_repo
+            self._loadable_repository_symbols.append(
+                LoadableRepositorySymbol(
+                    attribute=loadable_target.attribute,
+                    repository_name=repo_def.name,
+                )
+            )
 
     @property
     def loadable_repository_symbols(self):
@@ -102,56 +126,29 @@ class RepositorySymbolsAndCodePointers:
     def code_pointers_by_repo_name(self):
         return self._code_pointers_by_repo_name
 
+    def get_recon_repo(self, name: str) -> ReconstructableRepository:
+        return self._recon_repos_by_name[name]
 
-def load_loadable_repository_symbols(loadable_target_origin):
-    if loadable_target_origin:
-        loadable_targets = get_loadable_targets(
+
+def _get_code_pointer(loadable_target_origin, loadable_repository_symbol):
+    if loadable_target_origin.python_file:
+        return CodePointer.from_python_file(
             loadable_target_origin.python_file,
-            loadable_target_origin.module_name,
+            loadable_repository_symbol.attribute,
+            loadable_target_origin.working_directory,
+        )
+    elif loadable_target_origin.package_name:
+        return CodePointer.from_python_package(
             loadable_target_origin.package_name,
+            loadable_repository_symbol.attribute,
             loadable_target_origin.working_directory,
-            loadable_target_origin.attribute,
         )
-        return [
-            LoadableRepositorySymbol(
-                attribute=loadable_target.attribute,
-                repository_name=repository_def_from_target_def(
-                    loadable_target.target_definition
-                ).name,
-            )
-            for loadable_target in loadable_targets
-        ]
     else:
-        return []
-
-
-def build_code_pointers_by_repo_name(loadable_target_origin, loadable_repository_symbols):
-    repository_code_pointer_dict = {}
-    for loadable_repository_symbol in loadable_repository_symbols:
-        if loadable_target_origin.python_file:
-            repository_code_pointer_dict[
-                loadable_repository_symbol.repository_name
-            ] = CodePointer.from_python_file(
-                loadable_target_origin.python_file,
-                loadable_repository_symbol.attribute,
-                loadable_target_origin.working_directory,
-            )
-        elif loadable_target_origin.package_name:
-            repository_code_pointer_dict[
-                loadable_repository_symbol.repository_name
-            ] = CodePointer.from_python_package(
-                loadable_target_origin.package_name,
-                loadable_repository_symbol.attribute,
-            )
-        else:
-            repository_code_pointer_dict[
-                loadable_repository_symbol.repository_name
-            ] = CodePointer.from_module(
-                loadable_target_origin.module_name,
-                loadable_repository_symbol.attribute,
-            )
-
-    return repository_code_pointer_dict
+        return CodePointer.from_module(
+            loadable_target_origin.module_name,
+            loadable_repository_symbol.attribute,
+            loadable_target_origin.working_directory,
+        )
 
 
 class DagsterApiServer(DagsterApiServicer):
@@ -166,6 +163,7 @@ class DagsterApiServer(DagsterApiServicer):
         heartbeat_timeout=30,
         lazy_load_user_code=False,
         fixed_server_id=None,
+        entry_point=None,
     ):
         super(DagsterApiServer, self).__init__()
 
@@ -180,6 +178,8 @@ class DagsterApiServer(DagsterApiServicer):
             loadable_target_origin, "loadable_target_origin", LoadableTargetOrigin
         )
 
+        self._mp_ctx = multiprocessing.get_context("spawn")
+
         # Each server is initialized with a unique UUID. This UUID is used by clients to track when
         # servers are replaced and is used for cache invalidation and reloading.
         self._server_id = check.opt_str_param(fixed_server_id, "fixed_server_id", str(uuid.uuid4()))
@@ -198,14 +198,20 @@ class DagsterApiServer(DagsterApiServicer):
 
         self._serializable_load_error = None
 
-        self._repository_symbols_and_code_pointers = RepositorySymbolsAndCodePointers(
-            loadable_target_origin
+        self._entry_point = (
+            frozenlist(check.list_param(entry_point, "entry_point", of_type=str))
+            if entry_point != None
+            else DEFAULT_DAGSTER_ENTRY_POINT
         )
+
         try:
-            self._repository_symbols_and_code_pointers.load()
-        except Exception:  # pylint:disable=broad-except
+            self._loaded_repositories = LoadedRepositories(
+                loadable_target_origin, self._entry_point
+            )
+        except Exception:
             if not lazy_load_user_code:
                 raise
+            self._loaded_repositories = None
             self._serializable_load_error = serializable_error_info_from_exc_info(sys.exc_info())
 
         self.__last_heartbeat_time = time.time()
@@ -286,24 +292,13 @@ class DagsterApiServer(DagsterApiServicer):
         if run_id in self._termination_times:
             del self._termination_times[run_id]
 
-    def _recon_repository_from_origin(self, external_repository_origin):
-        check.inst_param(
-            external_repository_origin,
-            "external_repository_origin",
-            ExternalRepositoryOrigin,
-        )
+    def _recon_repository_from_origin(
+        self, external_repository_origin: ExternalRepositoryOrigin
+    ) -> ReconstructableRepository:
+        # could assert against external_repository_origin.repository_location_origin
+        return self._loaded_repositories.get_recon_repo(external_repository_origin.repository_name)
 
-        return ReconstructableRepository(
-            self._repository_symbols_and_code_pointers.code_pointers_by_repo_name[
-                external_repository_origin.repository_name
-            ],
-            self._get_current_image(),
-        )
-
-    def _recon_pipeline_from_origin(self, external_pipeline_origin):
-        check.inst_param(
-            external_pipeline_origin, "external_pipeline_origin", ExternalPipelineOrigin
-        )
+    def _recon_pipeline_from_origin(self, external_pipeline_origin: ExternalPipelineOrigin):
         recon_repo = self._recon_repository_from_origin(
             external_pipeline_origin.external_repository_origin
         )
@@ -352,13 +347,12 @@ class DagsterApiServer(DagsterApiServicer):
             )
 
         response = ListRepositoriesResponse(
-            self._repository_symbols_and_code_pointers.loadable_repository_symbols,
+            self._loaded_repositories.loadable_repository_symbols,
             executable_path=self._loadable_target_origin.executable_path
             if self._loadable_target_origin
             else None,
-            repository_code_pointer_dict=(
-                self._repository_symbols_and_code_pointers.code_pointers_by_repo_name
-            ),
+            repository_code_pointer_dict=self._loaded_repositories.code_pointers_by_repo_name,
+            entry_point=self._entry_point,
         )
 
         return api_pb2.ListRepositoriesReply(
@@ -563,7 +557,7 @@ class DagsterApiServer(DagsterApiServicer):
                     ShutdownServerResult(success=True, serializable_error_info=None)
                 )
             )
-        except:  # pylint: disable=bare-except
+        except:
             return api_pb2.ShutdownServerReply(
                 serialized_shutdown_server_result=serialize_dagster_namedtuple(
                     ShutdownServerResult(
@@ -590,7 +584,7 @@ class DagsterApiServer(DagsterApiServicer):
                     self._termination_times[cancel_execution_request.run_id] = time.time()
                     success = True
 
-        except:  # pylint: disable=bare-except
+        except:
             serializable_error_info = serializable_error_info_from_exc_info(sys.exc_info())
 
         return api_pb2.CancelExecutionReply(
@@ -640,7 +634,7 @@ class DagsterApiServer(DagsterApiServicer):
             run_id = execute_run_args.pipeline_run_id
             recon_pipeline = self._recon_pipeline_from_origin(execute_run_args.pipeline_origin)
 
-        except:  # pylint: disable=bare-except
+        except:
             return api_pb2.StartRunReply(
                 serialized_start_run_result=serialize_dagster_namedtuple(
                     StartRunResult(
@@ -653,9 +647,9 @@ class DagsterApiServer(DagsterApiServicer):
                 )
             )
 
-        event_queue = multiprocessing.Queue()
-        termination_event = multiprocessing.Event()
-        execution_process = multiprocessing.Process(
+        event_queue = self._mp_ctx.Queue()
+        termination_event = self._mp_ctx.Event()
+        execution_process = self._mp_ctx.Process(
             target=start_run_in_subprocess,
             args=[
                 request.serialized_execute_run_args,
@@ -727,19 +721,20 @@ class DagsterApiServer(DagsterApiServicer):
             )
         )
 
-    def _get_current_image(self):
-        return os.getenv("DAGSTER_CURRENT_IMAGE")
-
     def GetCurrentImage(self, request, _context):
         return api_pb2.GetCurrentImageReply(
             serialized_current_image=serialize_dagster_namedtuple(
                 GetCurrentImageResult(
-                    current_image=self._get_current_image(), serializable_error_info=None
+                    current_image=_get_current_image(), serializable_error_info=None
                 )
             )
         )
 
 
+def _get_current_image():
+    return os.getenv("DAGSTER_CURRENT_IMAGE")
+
+
 @whitelist_for_serdes
 class GrpcServerStartedEvent(namedtuple("GrpcServerStartedEvent", "")):
     pass
@@ -778,6 +773,7 @@ class DagsterGrpcServer:
         lazy_load_user_code=False,
         ipc_output_file=None,
         fixed_server_id=None,
+        entry_point=None,
     ):
         check.opt_str_param(host, "host")
         check.opt_int_param(port, "port")
@@ -811,6 +807,10 @@ class DagsterGrpcServer:
         self.server = grpc.server(
             ThreadPoolExecutor(max_workers=max_workers),
             compression=grpc.Compression.Gzip,
+            options=[
+                ("grpc.max_send_message_length", max_send_bytes()),
+                ("grpc.max_receive_message_length", max_rx_bytes()),
+            ],
         )
         self._server_termination_event = threading.Event()
 
@@ -822,6 +822,7 @@ class DagsterGrpcServer:
                 heartbeat_timeout=heartbeat_timeout,
                 lazy_load_user_code=lazy_load_user_code,
                 fixed_server_id=fixed_server_id,
+                entry_point=entry_point,
             )
         except Exception:
             if self._ipc_output_file:
@@ -931,7 +932,7 @@ def wait_for_grpc_server(server_process, client, subprocess_args, timeout=60):
         try:
             client.ping("")
             return
-        except grpc._channel._InactiveRpcError:  # pylint: disable=protected-access
+        except DagsterUserCodeUnreachableError:
             last_error = serializable_error_info_from_exc_info(sys.exc_info())
 
         if time.time() - start_time > timeout:
@@ -965,14 +966,15 @@ def open_server_process(
 
     mocked_system_timezone = get_mocked_system_timezone()
 
+    executable_path = loadable_target_origin.executable_path if loadable_target_origin else None
+
     subprocess_args = (
-        [
-            loadable_target_origin.executable_path
-            if loadable_target_origin and loadable_target_origin.executable_path
-            else sys.executable,
-            "-m",
-            "dagster.grpc",
-        ]
+        (
+            get_python_environment_entry_point(executable_path)
+            if executable_path
+            else DEFAULT_DAGSTER_ENTRY_POINT
+        )
+        + ["api", "grpc"]
         + ["--lazy-load-user-code"]
         + (["--port", str(port)] if port else [])
         + (["--socket", socket] if socket else [])
@@ -981,6 +983,8 @@ def open_server_process(
         + (["--heartbeat-timeout", str(heartbeat_timeout)] if heartbeat_timeout else [])
         + (["--fixed-server-id", fixed_server_id] if fixed_server_id else [])
         + (["--override-system-timezone", mocked_system_timezone] if mocked_system_timezone else [])
+        + (["--log-level", "WARNING"])  # don't log INFO messages for automatically spun up servers
+        + (["--use-python-environment-entry-point"] if executable_path else [])
     )
 
     if loadable_target_origin:
